# Result of joining the database schema tables
METADATA = {
    "dty_ID": 1111,
    "rst_DisplayName": "date_of_creation",
    "dty_Type": "date",
    "rst_MaxValues": 1,
}

# Result of a record's JSON export
DETAIL = {
    "dty_ID": 1111,
    "value": {
        "start": {
            "earliest": "1180",
            "latest": "1231",
            "profile": "1",
        },
        "end": {
            "latest": "1250",
            "earliest": "1246",
            "profile": "3",
        },
        "determination": "2",
        "estMinDate": 1180,
        "estMaxDate": 1250.1231,
    },
    "fieldName": "date / time",
    "fieldType": "date",
    "conceptID": "",
}
