# Result of joining the database schema tables
METADATA = {
    "dty_ID": 1109,
    "rst_DisplayName": "preferred_name",
    "dty_Type": "freetext",
    "rst_MaxValues": 1,
}

# Result of a record's JSON export
DETAIL = {
    "dty_ID": 1109,
    "value": "name of something",
    "fieldName": "single line",
    "fieldType": "freetext",
    "conceptID": "",
}
